import numpy as np
import matplotlib.pyplot as plt
from matplotlib import cm as cm
import re
from sklearn.metrics.pairwise import cosine_similarity
import sklearn.feature_extraction
import nltk.stem.porter

sen = [
    "Tim loves the band Korn.",
    "Tim adores the rock group Korn.",
    "Tim loves eating corn.",
    "Tim used to love Korn, but now he hates them.",
    "Tim absolutely loves Korn.",
    "Tim completely detests the performers named Korn",
    "Tim has a deep passion for the outfit the goes by the name of Korn",
    "Tim loves listening to the band Korn while eating corn."
]

np.set_printoptions(precision=2)

def jaccard(s1, s2):
    print(s1)
    print(s2)
    print(s1.intersection(s2))
    j = float(len(s1.intersection(s2))) / float(len(s1.union(s2)))
    print(j)
    return j

def plot_sim_matrix(m,sens):
    cmap = cm.get_cmap('RdYlGn')
    fig, ax = plt.subplots(figsize=(8,8))
    cax = ax.matshow(m, interpolation='nearest', cmap=cmap)
    ax.grid(True)
    plt.title('San Francisco Similarity matrix')
    plt.xticks(range(len(sens)), sens, rotation=90);
    plt.yticks(range(len(sens)), sens);
    fig.colorbar(cax, ticks=[0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, .75,.8,.85,.90,.95,1])
    plt.show()

#stemmer = nltk.stem.porter.PorterStemmer()
#for w in sen[0].split(" "):
#    print(stemmer.stem(w))


#from nltk.corpus import stopwords
#print(stopwords.words('english'))


def stemmed_words(doc):
    s = (stemmer.stem(w) for w in analyzer(doc))
    for w in s:
        print(w)
    return (stemmer.stem(w) for w in analyzer(doc))

analyzer = sklearn.feature_extraction.text.CountVectorizer(stop_words='english').build_analyzer()

def do_count_vectorization():

    f = sklearn.feature_extraction.text.CountVectorizer(analyzer=stemmed_words)
    

    #Count vectorizer translates each document into a vector of counts
    #f = sklearn.feature_extraction.text.CountVectorizer()
    X = f.fit_transform(sen)

    print(X.toarray())
    print(f.get_feature_names())


    #cosine_similarity computes the cosine similarity between
    #a set of vectors
    from sklearn.metrics.pairwise import cosine_similarity
    cos_sim = cosine_similarity(X)

    print(cos_sim)



    plot_sim_matrix(cos_sim, sen)

#do_count_vectorization()

#now do tfidf

def do_tf_idf():
    #from scratch
    f = sklearn.feature_extraction.text.CountVectorizer(analyzer=stemmed_words)
    X = f.fit_transform(sen)
    cnts_ar = np.array(X.toarray()).sum(axis=0)

    doc_len_ar = np.array(X.toarray()).sum(axis=1)
    #should count number of documents where term appears, not just sum, 
    # but ok because our corpus doesn't repeat words
    idf_ar = np.log((1.0/cnts_ar) * len(sen)) + 1
    tdf_out = (np.array(X.toarray()).T/(doc_len_ar)).T * idf_ar
    print(X.toarray())
    print(cnts_ar)
    print((np.array(X.toarray()).T/(doc_len_ar)).T)
    print(tdf_out)
    cos_sim = cosine_similarity(
        tdf_out
        )
    #plot_sim_matrix(cos_sim, sen)

    #using sklearn
    f  = sklearn.feature_extraction.text.TfidfVectorizer(analyzer=stemmed_words,smooth_idf=False,norm='l1')
    X = f.fit_transform(sen)
    print(X.toarray())
    cos_sim = cosine_similarity(
        X
        )
    print(cos_sim)
    plot_sim_matrix(cos_sim, sen)

#do_tf_idf()

def do_jaccard():
    out = np.zeros((len(sen),len(sen)))
    print(out)
    for i in range(len(sen)):
        s = re.sub(r'[^\w\s]', '', sen[i])
        sen1 = set(s.split(" "))
        for j in range(len(sen)):
            s = re.sub(r'[^\w\s]', '', sen[j])
            sen2 = set(s.split(" "))

            out[i][j] = jaccard(sen1, sen2)

    print(out)
    plot_sim_matrix(out, sen)

#do_jaccard()

def do_bert():

    from sentence_transformers import SentenceTransformer

    model = SentenceTransformer('all-mpnet-base-v2')
    sen_embeddings = model.encode(sen)

    print(sen_embeddings[0])
    print(len(sen_embeddings[0]))

    from sklearn.metrics.pairwise import cosine_similarity
    #let's calculate cosine similarity for sentence 0:
    cos_sim = cosine_similarity(sen_embeddings)

    print(cos_sim)
    plot_sim_matrix(cos_sim, sen)

do_bert()